@@ -33,23 +33,26 @@ werrors="$werrors -Werror=implicit-function-declaration"
 setup_base()
 {
   # use default python flags but remove sign-compare
-  sysflags="$($PYTHON -c "from distutils import sysconfig; \
+  sysflags="$($PYTHON -c "import sysconfig; \
     print (sysconfig.get_config_var('CFLAGS'))")"
   export CFLAGS="$sysflags $werrors -Wlogical-op -Wno-sign-compare"
-  # SIMD extensions that need to be tested on both runtime and compile-time via (test_simd.py)
-  # any specified features will be ignored if they're not supported by compiler or platform
-  # note: it almost the same default value of --simd-test execpt adding policy `$werror` to treat all
-  # warnings as errors
-  simd_test="\$werror BASELINE SSE2 SSE42 XOP FMA4 (FMA3 AVX2) AVX512F AVX512_SKX VSX VSX2 VSX3 NEON ASIMD"
-  # We used to use 'setup.py install' here, but that has the terrible
-  # behaviour that if a copy of the package is already installed in the
-  # install location, then the new copy just gets dropped on top of it.
-  # Travis typically has a stable numpy release pre-installed, and if we
-  # don't remove it, then we can accidentally end up e.g. running old
-  # test modules that were in the stable release but have been removed
-  # from main. (See gh-2765, gh-2768.)  Using 'pip install' also has
-  # the advantage that it tests that numpy is 'pip install' compatible,
-  # see e.g. gh-2766...
+
+  build_args=()
+  # Strictly disable all kinds of optimizations
+  if [ -n "$WITHOUT_OPTIMIZATIONS" ]; then
+      build_args+=("--disable-optimization")
+  # only disable SIMD optimizations
+  elif [ -n "$WITHOUT_SIMD" ]; then
+      build_args+=("--cpu-baseline=none" "--cpu-dispatch=none")
+  elif [ -n "$CPU_DISPATCH" ]; then
+      build_args+=("--cpu-dispatch=$CPU_DISPATCH")
+  else
+    # SIMD extensions that need to be tested on both runtime and compile-time via (test_simd.py)
+    # any specified features will be ignored if they're not supported by compiler or platform
+    # note: it almost the same default value of --simd-test execpt adding policy `$werror` to treat all
+    # warnings as errors
+    build_args+=("--simd-test=\$werror BASELINE SSE2 SSE42 XOP FMA4 (FMA3 AVX2) AVX512F AVX512_SKX VSX VSX2 VSX3 NEON ASIMD VX VXE VXE2")
+  fi
   if [ -z "$USE_DEBUG" ]; then
     # activates '-Werror=undef' when DEBUG isn't enabled since _cffi_backend'
     # extension breaks the build due to the following error:
@@ -58,11 +61,11 @@ setup_base()
     # #if !HAVE_FFI_PREP_CIF_VAR && defined(__arm64__) && defined(__APPLE__)
     #
     export CFLAGS="$CFLAGS -Werror=undef"
-    $PYTHON setup.py build --simd-test "$simd_test" install 2>&1 | tee log
+    $PYTHON setup.py build "${build_args[@]}" install 2>&1 | tee log
   else
     # The job run with USE_DEBUG=1 on travis needs this.
     export CFLAGS=$CFLAGS" -Wno-maybe-uninitialized"
-    $PYTHON setup.py build --simd-test "$simd_test" build_src --verbose-cfg build_ext --inplace 2>&1 | tee log
+    $PYTHON setup.py build "${build_args[@]}" build_src --verbose-cfg build_ext --inplace 2>&1 | tee log
   fi
   grep -v "_configtest" log \
     | grep -vE "ld returned 1|no files found matching" \
@@ -83,7 +86,7 @@ run_test()
   # in test_requirements.txt) does not provide a wheel, and the source tar
   # file does not install correctly when Python's optimization level is set
   # to strip docstrings (see https://github.com/eliben/pycparser/issues/291).
-  PYTHONOPTIMIZE="" $PIP install -r test_requirements.txt
+  PYTHONOPTIMIZE="" $PIP install -r test_requirements.txt pyinstaller
   DURATIONS_FLAG="--durations 10"
 
   if [ -n "$USE_DEBUG" ]; then
@@ -103,15 +106,54 @@ run_test()
     "import os; import numpy; print(os.path.dirname(numpy.__file__))")
   export PYTHONWARNINGS=default
 
+  # This guard protects against any sudden unexpected changes that may adversely
+  # affect the compile-time SIMD features detection which could leave the SIMD code
+  # inactivated. Until now we have faced two cases:
+  #
+  # 1. Hardening the compile-time test files of Neon/ASIMD features without checking
+  # the sanity of the modification leading to disabling all optimizations on aarch64.
+  # see gh-21747
+  #
+  # 2. A sudden compiler upgrades by CI side on s390x that causes conflicts with the
+  # installed assembler leading to disabling the whole VX/E features, which made us
+  # merge SIMD code without testing it. Later, it was discovered that this code
+  # disrupted the NumPy build.
+  # see gh-21750, gh-21748
+  if [ -n "$EXPECT_CPU_FEATURES" ]; then
+    as_expected=$($PYTHON << EOF
+from numpy.core._multiarray_umath import (__cpu_baseline__, __cpu_dispatch__)
+features = __cpu_baseline__ + __cpu_dispatch__
+expected = '$EXPECT_CPU_FEATURES'.upper().split()
+diff = set(expected).difference(set(features))
+if diff:
+    print("Unexpected compile-time CPU features detection!\n"
+          f"The current build is missing the support of CPU features '{diff}':\n"
+          "This error is triggered because of one of the following reasons:\n\n"
+          f"1. The compiler for somehow no longer supports any of these CPU features {diff}\n"
+          f"Note that the current build reports the support of the following features:\n{features}\n\n"
+          "2. Your code messed up the testing process! please check the build log and trace\n"
+          "compile-time feature tests and make sure that your patch has nothing to do with the tests failures."
+          )
+EOF
+)
+    if [ -n "$as_expected" ]; then
+      echo "$as_expected"
+      exit 1
+    fi
+  fi
+
   if [ -n "$CHECK_BLAS" ]; then
+    $PYTHON -m pip install threadpoolctl
     $PYTHON ../tools/openblas_support.py --check_version
   fi
 
   if [ -n "$RUN_FULL_TESTS" ]; then
+    # Travis has a limit on log length that is causeing test failutes.
+    # The fix here is to remove the "-v" from the runtest arguments.
     export PYTHONWARNINGS="ignore::DeprecationWarning:virtualenv"
-    $PYTHON -b ../runtests.py -n -v --mode=full $DURATIONS_FLAG $COVERAGE_FLAG
+    $PYTHON -b ../runtests.py -n --mode=full $DURATIONS_FLAG $COVERAGE_FLAG
   else
-    $PYTHON ../runtests.py -n -v $DURATIONS_FLAG -- -rs
+    $PYTHON ../runtests.py -n $DURATIONS_FLAG -- -rs
   fi
 
   if [ -n "$RUN_COVERAGE" ]; then
@@ -139,7 +181,7 @@ run_test()
     pushd ../benchmarks
     $PYTHON `which asv` check --python=same
     $PYTHON `which asv` machine --machine travis
-    $PYTHON `which asv` dev 2>&1| tee asv-output.log
+    $PYTHON `which asv` dev -q 2>&1| tee asv-output.log
     if grep -q Traceback asv-output.log; then
       echo "Some benchmarks have errors!"
       exit 1
